import os
import logging
import re

import aexpect
from aexpect import remote

from avocado.utils import download
from avocado.utils import aurl
from avocado.utils import wait

from six.moves import xrange

from . import data_dir
from . import remote as remote_old


class NetperfError(Exception):
    pass


class NetperfPackageError(NetperfError):

    def __init__(self, error_info):
        NetperfError.__init__(self)
        self.error_info = error_info

    def __str__(self):
        e_msg = "Packeage Error: %s" % self.error_info
        return e_msg


class NetserverError(NetperfError):

    def __init__(self, error_info):
        NetperfError.__init__(self)
        self.error_info = error_info

    def __str__(self):
        e_msg = "%s" % self.error_info
        return e_msg


class NetperfTestError(NetperfError):

    def __init__(self, error_info):
        NetperfError.__init__(self)
        self.error_info = error_info

    def __str__(self):
        e_msg = "Netperf test error: %s" % self.error_info
        return e_msg


class NetperfPackage(remote_old.Remote_Package):

    def __init__(self, address, netperf_path, md5sum="", netperf_source="",
                 client="ssh", port="22", username="root", password="123456",
                 prompt="^root@.*[\#\$]\s*$|", linesep="\n", status_test_command="echo $?"):
        """
        Class NetperfPackage just represent the netperf package
        Init NetperfPackage class.

        :param address: Remote host or guest address
        :param netperf_path: Installed netperf path
        :param me5sum: Local netperf package me5sum
        :param netperf_source: source netperf (path or link) path
        :param client: The client to use ('ssh', 'telnet' or 'nc')
        :param port: Port to connect to
        :param username: Username (if required)
        :param password: Password (if required)
        """
        super(NetperfPackage, self).__init__(address, client, username,
                                             password, port, netperf_path)

        self.netperf_source = netperf_source
        self.pack_suffix = ""
        self.netperf_dir = None
        self.build_tool = False
        self.md5sum = md5sum
        self.netperf_base_dir = self.remote_path
        self.netperf_file = os.path.basename(self.netperf_source)

        if client == "ssh":
            if self.netperf_source.endswith("tar.bz2"):
                self.pack_suffix = ".tar.bz2"
                self.decomp_cmd = "tar jxf"
            elif self.netperf_source.endswith("tar.gz"):
                self.pack_suffix = ".tar.gz"
                self.decomp_cmd = "tar zxf"
            self.netperf_dir = os.path.join(self.remote_path,
                                            self.netperf_file.rstrip(self.pack_suffix))

        if self.pack_suffix:
            self.netserver_path = os.path.join(self.netperf_dir,
                                               "src/netserver")
            self.netperf_path = os.path.join(self.netperf_dir,
                                             "src/netperf")
        else:
            self.netserver_path = os.path.join(self.netperf_base_dir,
                                               self.netperf_file)
            self.netperf_path = os.path.join(self.netperf_base_dir,
                                             self.netperf_file)

        logging.debug("Create remote session")
        self.session = remote.remote_login(self.client, self.address,
                                           self.port, self.username,
                                           self.password, prompt,
                                           linesep, timeout=360,
                                           status_test_command=status_test_command)

    def env_cleanup(self, clean_all=True):
        clean_cmd = ""
        if self.netperf_dir:
            clean_cmd = "rm -rf %s" % self.netperf_dir
        if clean_all:
            clean_cmd += " rm -rf %s" % os.path.join(self.remote_path,
                                                     self.netperf_file)
        self.session.cmd(clean_cmd, ignore_all_errors=True)

    def pack_compile(self, compile_option=""):
        pre_setup_cmd = "cd %s " % self.netperf_base_dir
        pre_setup_cmd += " && %s %s" % (self.decomp_cmd, self.netperf_file)
        pre_setup_cmd += " && cd %s " % self.netperf_dir
        # Create dict to make other OS architectures easy to extend
        build_type = {"aarch64": "aarch64-unknown-linux-gnu"}
        build_arch = self.session.cmd_output("arch", timeout=60).strip()
        np_build = build_type.get(build_arch, build_arch).strip()
        setup_cmd = "./autogen.sh > /dev/null 2>&1 &&" \
                    " ./configure --build=%s %s > /dev/null 2>&1" % (np_build,
                                                                     compile_option)
        setup_cmd += " && make > /dev/null 2>&1"
        self.env_cleanup(clean_all=False)
        cmd = "%s && %s " % (pre_setup_cmd, setup_cmd)
        try:
            self.session.cmd(cmd, timeout=1200)
        except aexpect.ShellError as e:
            raise NetperfPackageError("Compile failed: %s" % e)

    def pull_file(self, netperf_source=None):
        """
        Copy file from remote to local.
        """

        if aurl.is_url(netperf_source):
            logging.debug("Download URL file to local path")
            tmp_dir = data_dir.get_download_dir()
            dst = os.path.join(tmp_dir, os.path.basename(netperf_source))
            self.netperf_source = download.get_file(src=netperf_source,
                                                    dst=dst,
                                                    hash_expected=self.md5sum)
        else:
            self.netperf_source = netperf_source
        return self.netperf_source

    def install(self, install, compile_option):
        cmd = "which netperf"
        try:
            status, netperf = self.session.cmd_status_output(cmd)
        except aexpect.ShellError:
            status = 1
        if not status:
            self.netperf_path = netperf.rstrip()
            cmd = "which netserver"
            self.netserver_path = self.session.cmd_output(cmd).rstrip()
            install = False
        if install:
            self.build_tool = True
            self.pull_file(self.netperf_source)
            self.push_file(self.netperf_source)
            if self.pack_suffix:
                logging.debug("Compiling netserver from source")
                self.pack_compile(compile_option)

        msg = "Using local netperf: %s and %s" % (self.netperf_path,
                                                  self.netserver_path)
        logging.debug(msg)
        return (self.netserver_path, self.netperf_path)


class Netperf(object):

    def __init__(self, address, netperf_path, md5sum="", netperf_source="",
                 client="ssh", port="22", username="root", password="redhat",
                 prompt="^root@.*[\#\$]\s*$|", linesep="\n", status_test_command="echo $?",
                 compile_option="--enable-demo=yes", install=True):
        """
        Init Netperf class.

        :param address: Remote host or guest address
        :param netperf_path: Remote netperf path
        :param me5sum: Local netperf package me5sum
        :param netperf_source: netperf source file (path or link) which will
                               transfer to remote
        :param client: The client to use ('ssh', 'telnet' or 'nc')
        :param port: Port to connect to
        :param username: Username (if required)
        :param password: Password (if required)
        :param compile_option: Compile option for netperf
        :param install: Whether need install netperf or not.
        """
        self.client = client

        self.package = NetperfPackage(address, netperf_path, md5sum,
                                      netperf_source, client, port, username,
                                      password, prompt, linesep, status_test_command)
        self.netserver_path, self.netperf_path = self.package.install(install,
                                                                      compile_option)
        logging.debug("Create remote session")
        self.session = remote.remote_login(client, address, port, username,
                                           password, prompt,
                                           linesep, timeout=360,
                                           status_test_command=status_test_command)

    def is_target_running(self, target):
        list_cmd = "ps -C %s" % target
        if self.client == "nc":
            list_cmd = "wmic process where name='%s' list" % target
        try:
            output = self.session.cmd_output_safe(list_cmd, timeout=120)
            check_reg = re.compile(r"%s" % target, re.I | re.M)
            return bool(check_reg.findall(output))
        except Exception as err:
            logging.debug("Check process error: %s" % str(err))
        return False

    def stop(self, target):
        if self.client == "nc":
            stop_cmd = "taskkill /F /IM %s*" % target
        else:
            stop_cmd = "killall %s" % target
        if self.is_target_running(target):
            self.session.cmd(stop_cmd, ignore_all_errors=True)
        if self.is_target_running(target):
            raise NetserverError("Cannot stop %s" % target)
        logging.info("Stop %s successfully" % target)

    def cleanup(self, clean_all=True):
        """
        Cleanup the netperf packages.

        :param clean_all: True to delete both netperf binary and source tarball
                          and False to delete only binary.
        """
        self.package.env_cleanup(clean_all=clean_all)


class NetperfServer(Netperf):

    def __init__(self, address, netperf_path, md5sum="", netperf_source="",
                 client="ssh", port="22", username="root", password="redhat",
                 prompt="^root@.*[\#\$]\s*$|", linesep="\n", status_test_command="echo $?",
                 compile_option="--enable-demo=yes", install=True):
        """
        Init NetperfServer class.

        :param address: Remote host or guest address
        :param netperf_path: Remote netperf path
        :param me5sum: Local netperf package me5sum
        :param netperf_source: Local netperf (path or link) with will transfer to
                           remote
        :param client: The client to use ('ssh', 'telnet' or 'nc')
        :param port: Port to connect to
        :param username: Username (if required)
        :param password: Password (if required)
        :param compile_option: Compile option for netperf
        :param install: Whether need install netperf or not.
        """
        super(NetperfServer, self).__init__(address, netperf_path, md5sum,
                                            netperf_source, client, port, username,
                                            password, prompt, linesep, status_test_command,
                                            compile_option, install)

    def start(self, restart=False):
        """
        Start/Restart netserver

        :param restart: if restart=True, will restart the netserver
        """

        logging.info("Start netserver ...")
        server_cmd = ""
        if self.client == "nc":
            server_cmd += "start /b %s > null" % self.netserver_path
        else:
            server_cmd = "%s > /dev/null" % self.netserver_path

        if restart:
            self.stop()
        if not self.is_server_running():
            logging.info("Start netserver with cmd: '%s'" % server_cmd)
            self.session.cmd_output_safe(server_cmd)

        if not wait.wait_for(self.is_server_running, 5):
            raise NetserverError("Can not start netperf server!")
        logging.info("Netserver start successfully")

    def is_server_running(self):
        return self.is_target_running(os.path.basename(self.netserver_path))

    def stop(self):
        super(NetperfServer, self).stop(os.path.basename(self.netserver_path))


class NetperfClient(Netperf):

    def __init__(self, address, netperf_path, md5sum="", netperf_source="",
                 client="ssh", port="22", username="root", password="redhat",
                 prompt="^root@.*[\#\$]\s*$|", linesep="\n", status_test_command="echo $?",
                 compile_option="", install=True):
        """
        Init NetperfClient class.

        :param address: Remote host or guest address
        :param netperf_path: Remote netperf path
        :param me5sum: Local netperf package me5sum
        :param netperf_source: Netperf source file (path or link) which will
                               transfer to remote
        :param client: The client to use ('ssh', 'telnet' or 'nc')
        :param port: Port to connect to
        :param username: Username (if required)
        :param password: Password (if required)
        :param compile_option: Compile option for netperf
        :param install: Whether need install netperf or not.
        """
        super(NetperfClient, self).__init__(address, netperf_path, md5sum,
                                            netperf_source, client, port, username,
                                            password, prompt, linesep, status_test_command,
                                            compile_option, install)

    def start(self, server_address, test_option="", timeout=1200,
              cmd_prefix="", package_sizes=""):
        """
        Run netperf test

        :param server_address: Remote netserver address
        :param netperf_path: Netperf test option (global/test option)
        :param timeout: Netperf test timeout(-l)
        :param cmd_prefix: Prefix in netperf command
        :param package_sizes: Package sizes test in netperf command.
        :return: return test result
        """
        netperf_cmd = "%s %s -H %s %s " % (cmd_prefix, self.netperf_path,
                                           server_address, test_option)
        try:
            output = ""
            if package_sizes:
                for p_size in package_sizes.split():
                    cmd = netperf_cmd + " -- -m %s" % p_size
                    logging.info("Start netperf with cmd: '%s'" % cmd)
                    output += self.session.cmd_output_safe(cmd,
                                                           timeout=timeout)
            else:
                logging.info("Start netperf with cmd: '%s'" % netperf_cmd)
                output = self.session.cmd_output_safe(netperf_cmd,
                                                      timeout=timeout)
        except aexpect.ShellError as err:
            raise NetperfTestError("Run netperf error. %s" % str(err))
        self.result = output
        return self.result

    def bg_start(self, server_address, test_option="", session_num=1,
                 cmd_prefix="", package_sizes=""):
        """
        Run netperf background, for stress test do not have output

        :param server_address: Remote netserver address
        :param netperf_path: netperf test option (global/test option)
        :param timeout: Netperf test timeout(-l)
        :param cmd_prefix: Prefix in netperf command
        :param package_sizes: Package sizes test in netperf command.

        """
        if self.client == "nc":
            netperf_cmd = "start /b %s %s -H %s %s " % (cmd_prefix,
                                                        self.netperf_path,
                                                        server_address,
                                                        test_option)
        else:
            netperf_cmd = "%s %s -H %s %s " % (cmd_prefix,
                                               self.netperf_path,
                                               server_address,
                                               test_option)
        if package_sizes:
            for p_size in package_sizes.split():
                cmd = netperf_cmd + " -- -m %s" % p_size
                if self.client == "nc":
                    cmd = "%s > null " % cmd
                else:
                    cmd = "%s > /dev/null" % cmd
                txt = "Start %s sessions netperf background" % session_num
                txt += " with cmd: '%s' " % cmd
                logging.info(txt)
                for num in xrange(int(session_num)):
                    self.session.cmd_output_safe("%s &" % cmd)
        else:
            if self.client == "nc":
                netperf_cmd = "%s > null " % netperf_cmd
            else:
                netperf_cmd = "%s > /dev/null " % netperf_cmd
            txt = "Start %s sessions netperf background" % session_num
            txt += " with cmd: '%s' " % netperf_cmd
            logging.info(txt)
            for num in xrange(int(session_num)):
                self.session.cmd_output_safe("%s &" % netperf_cmd)

    def is_netperf_running(self):
        return self.is_target_running(os.path.basename(self.netperf_path))

    def stop(self):
        super(NetperfClient, self).stop(os.path.basename(self.netperf_path))



"""
Module to control libvirtd service.
"""
import re
import logging

import aexpect
from avocado.utils import path
from avocado.utils import process
from avocado.utils import wait

from virttest import libvirt_version
from virttest import utils_split_daemons

from . import remote as remote_old
from . import utils_misc
from .staging import service
from .utils_gdb import GDB

try:
    path.find_command("libvirtd")
    LIBVIRTD = "libvirtd"
except path.CmdNotFoundError:
    LIBVIRTD = None


class Libvirtd(object):

    """
    Class to manage libvirtd service on host or guest.
    """

    def __init__(self, service_name=None, session=None):
        """
        Initialize an service object for libvirtd.

        :params service_name: Service name such as virtqemud or libvirtd.
            If service_name is None, all sub daemons will be operated when
            modular daemon environment is enabled. Otherwise,if service_name is
            a single string, only the given daemon/service will be operated.
        :params session: An session to guest or remote host.
        """
        self.session = session
        if self.session:
            self.remote_runner = remote_old.RemoteRunner(session=self.session)
            runner = self.remote_runner.run
        else:
            runner = process.run

        self.daemons = []
        self.service_list = []

        if LIBVIRTD is None:
            logging.warning("Libvirtd service is not available in host, "
                            "utils_libvirtd module will not function normally")

        self.service_name = "libvirtd" if not service_name else service_name

        if libvirt_version.version_compare(5, 6, 0, self.session):
            if utils_split_daemons.is_modular_daemon(session=self.session):
                if self.service_name in ["libvirtd", "libvirtd.service"]:
                    self.service_list = ['virtqemud', 'virtproxyd',
                                         'virtnetworkd', 'virtinterfaced',
                                         'virtnodedevd', 'virtsecretd',
                                         'virtstoraged', 'virtnwfilterd']
                elif self.service_name == "libvirtd.socket":
                    self.service_name = "virtqemud.socket"
                elif self.service_name in ["libvirtd-tcp.socket", "libvirtd-tls.socket"]:
                    self.service_name = re.sub("libvirtd", "virtproxyd",
                                               self.service_name)
            else:
                self.service_name = re.sub("^virt.*d", "libvirtd",
                                           self.service_name)
        else:
            self.service_name = "libvirtd"
        if not self.service_list:
            self.service_list = [self.service_name]
        for serv in self.service_list:
            self.daemons.append(service.Factory.create_service(serv, run=runner))

    def _wait_for_start(self, timeout=60):
        """
        Wait n seconds for libvirt to start. Default is 10 seconds.
        """
        def _check_start():
            virsh_cmd = "virsh list"
            try:
                if self.session:
                    self.session.cmd(virsh_cmd, timeout=2)
                else:
                    process.run(virsh_cmd, timeout=2)
                return True
            except Exception:
                return False
        return utils_misc.wait_for(_check_start, timeout=timeout)

    def start(self, reset_failed=True):
        result = []
        for daem_item in self.daemons:
            if reset_failed:
                daem_item.reset_failed()
            if not daem_item.start():
                return False
            result.append(self._wait_for_start())
        return all(result)

    def stop(self):
        result = []
        for daem_item in self.daemons:
            result.append(daem_item.stop())
        return all(result)

    def restart(self, reset_failed=True):
        result = []
        for daem_item in self.daemons:
            if reset_failed:
                daem_item.reset_failed()
            if not daem_item.restart():
                return False
            result.append(self._wait_for_start())
        return all(result)

    def is_running(self):
        result = []
        for daem_item in self.daemons:
            result.append(daem_item.status())
        return all(result)


class DaemonSocket(object):

    """
    Class to manage libvirt/virtproxy tcp/tls socket on host or guest.
    """

    def __init__(self, daemon_name, session=None):
        """
        Initialize an service object for virt daemons.

        :param daemon_name: daemon name such as virtproxyd-tls.socket,
            libvirtd-tcp.socket,etc,.
        :param session: An session to guest or remote host.
        """
        self.session = session
        if self.session:
            self.remote_runner = remote_old.RemoteRunner(session=self.session)
            self.runner = self.remote_runner.run
        else:
            self.runner = process.run

        self.daemon_name = daemon_name
        supported_daemon = ["libvirtd-tcp.socket", "libvirtd-tls.socket",
                            "virtproxyd-tls.socket", "virtproxyd-tcp.socket"]
        if self.daemon_name not in supported_daemon:
            raise ValueError("Invalid daemon: %s" % self.daemon_name)

        self.daemon_service_inst = Libvirtd("virtproxyd", session=self.session)
        self.daemon_inst = Libvirtd(self.daemon_name, session=self.session)
        self.daemon_socket = Libvirtd("virtproxyd.socket", session=self.session)

    def stop(self):
        self.daemon_socket.stop()
        self.daemon_service_inst.stop()
        self.daemon_inst.stop()
        self.runner("systemctl daemon-reload")
        self.daemon_socket.start()

    def start(self):
        self.daemon_socket.stop()
        self.daemon_service_inst.stop()
        self.runner("systemctl daemon-reload")
        self.daemon_inst.start()
        self.daemon_service_inst.start()

    def restart(self, reset_failed=True):
        self.daemon_socket.stop()
        self.daemon_service_inst.stop()
        self.runner("systemctl daemon-reload")
        self.daemon_inst.restart()
        self.daemon_service_inst.start()
        self.daemon_inst._wait_for_start()


class LibvirtdSession(object):

    """
    Interaction daemon session by directly call the command.
    With gdb debugging feature can be optionally started.
    It is recommended to use the service in the modular daemons for
    initialization, because Libvirtd() class will switch to the
    corresponding service according to the environment,
    eg. If the value of "service_name" is "virtqemud",
    it will take "virtqemud" if the modular daemon is enabled
    and "libvirtd" if it's disabled.
    """

    def __init__(self, gdb=False,
                 logging_handler=None,
                 logging_params=(),
                 logging_pattern=r'.*',
                 service_name=None):
        """
        :param gdb: Whether call the session with gdb debugging support
        :param logging_handler: Callback function to handle logging
        :param logging_pattern: Regex for filtering specific log lines
        :param service_name: Service name such as virtqemud or libvirtd
        """
        self.gdb = None
        self.tail = None
        self.running = False
        self.pid = None
        self.service_name = service_name
        self.bundle = {"stop-info": None}
        # Get an executable program to debug by GDB
        self.service_exec = Libvirtd(
            service_name=self.service_name).service_list[0]
        self.libvirtd_service = Libvirtd(service_name=self.service_exec)
        self.was_running = self.libvirtd_service.is_running()
        if self.was_running:
            logging.debug('Stopping %s service', self.service_exec)
            self.libvirtd_service.stop()

        self.logging_handler = logging_handler
        self.logging_params = logging_params
        self.logging_pattern = logging_pattern

        if gdb:
            self.gdb = GDB(self.service_exec)
            self.gdb.set_callback('stop', self._stop_callback, self.bundle)
            self.gdb.set_callback('start', self._start_callback, self.bundle)
            self.gdb.set_callback('termination', self._termination_callback)

    def _output_handler(self, line):
        """
        Adapter output callback function.
        """
        if self.logging_handler is not None:
            if re.match(self.logging_pattern, line):
                self.logging_handler(line, *self.logging_params)

    def _termination_handler(self, status):
        """
        Helper aexpect terminaltion handler
        """
        self.running = False
        self.exit_status = status
        self.pid = None

    def _termination_callback(self, gdb, status):
        """
        Termination handler function triggered when libvirtd exited.

        :param gdb: Instance of the gdb session
        :param status: Return code of exited libvirtd session
        """
        self.running = False
        self.exit_status = status
        self.pid = None

    def _stop_callback(self, gdb, info, params):
        """
        Stop handler function triggered when gdb libvirtd stopped.

        :param gdb: Instance of the gdb session
        :param status: Return code of exited libvirtd session
        """
        self.running = False
        params['stop-info'] = info

    def _start_callback(self, gdb, info, params):
        """
        Stop handler function triggered when gdb libvirtd started.

        :param gdb: Instance of the gdb session
        :param status: Return code of exited libvirtd session
        """
        self.running = True
        params['stop-info'] = None

    def set_callback(self, callback_type, callback_func, callback_params=None):
        """
        Set a customized gdb callback function.
        """
        if self.gdb:
            self.gdb.set_callback(
                callback_type, callback_func, callback_params)
        else:
            logging.error("Only gdb session supports setting callback")

    def start(self, arg_str='', wait_for_working=True):
        """
        Start libvirtd session.

        :param arg_str: Argument passing to the session
        :param wait_for_working: Whether wait for libvirtd finish loading
        """
        if self.gdb:
            self.gdb.run(arg_str=arg_str)
            self.pid = self.gdb.pid
        else:
            self.tail = aexpect.Tail(
                "%s %s" % (self.service_exec, arg_str),
                output_func=self._output_handler,
                termination_func=self._termination_handler,
            )
            self.running = True

        if wait_for_working:
            self.wait_for_working()

    def cont(self):
        """
        Continue a stopped libvirtd session.
        """
        if self.gdb:
            self.gdb.cont()
        else:
            logging.error("Only gdb session supports continue")

    def kill(self):
        """
        Kill the libvirtd session.
        """
        if self.gdb:
            self.gdb.kill()
        else:
            self.tail.kill()

    def restart(self, arg_str='', wait_for_working=True):
        """
        Restart the libvirtd session.

        :param arg_str: Argument passing to the session
        :param wait_for_working: Whether wait for libvirtd finish loading
        """
        logging.debug("Restarting %s session", self.service_exec)
        self.kill()
        self.start(arg_str=arg_str, wait_for_working=wait_for_working)

    def wait_for_working(self, timeout=60):
        """
        Wait for libvirtd to work.

        :param timeout: Max wait time
        """
        logging.debug('Waiting for %s to work', self.service_exec)
        return utils_misc.wait_for(
            self.is_working,
            timeout=timeout,
        )

    def back_trace(self):
        """
        Get the backtrace from gdb session.
        """
        if self.gdb:
            return self.gdb.back_trace()
        else:
            logging.warning('Can not get back trace without gdb')

    def insert_break(self, break_func):
        """
        Insert a function breakpoint.

        :param break_func: Function at which breakpoint inserted
        """
        if self.gdb:
            return self.gdb.insert_break(break_func)
        else:
            logging.warning('Can not insert breakpoint without gdb')

    def is_working(self):
        """
        Check if libvirtd is start by return status of 'virsh list'
        """
        virsh_cmd = "virsh list"
        try:
            process.run(virsh_cmd, timeout=2)
            return True
        except process.CmdError:
            return False

    def wait_for_stop(self, timeout=60, step=0.1):
        """
        Wait for libvirtd to stop.

        :param timeout: Max wait time
        :param step: Checking interval
        """
        logging.debug('Waiting for %s to stop', self.service_exec)
        if self.gdb:
            return self.gdb.wait_for_stop(timeout=timeout)
        else:
            return wait.wait_for(
                lambda: not self.running,
                timeout=timeout,
                step=step,
            )

    def wait_for_termination(self, timeout=60):
        """
        Wait for libvirtd gdb session to exit.

        :param timeout: Max wait time
        """
        logging.debug('Waiting for %s to terminate', self.service_exec)
        if self.gdb:
            return self.gdb.wait_for_termination(timeout=timeout)
        else:
            logging.error("Only gdb session supports wait_for_termination.")

    def exit(self):
        """
        Exit the libvirtd session.
        """
        if self.gdb:
            self.gdb.exit()
        else:
            if self.tail:
                self.tail.close()

        if self.was_running:
            self.libvirtd_service.start()


def deprecation_warning():
    """
    As the utils_libvirtd.libvirtd_xxx interfaces are deprecated,
    this function are printing the warning to user.
    """
    logging.warning("This function was deprecated, Please use "
                    "class utils_libvirtd.Libvirtd to manage "
                    "libvirtd service.")


def libvirtd_start():
    libvirtd_instance = Libvirtd()
    deprecation_warning()
    return libvirtd_instance.start()


def libvirtd_is_running():
    libvirtd_instance = Libvirtd()
    deprecation_warning()
    return libvirtd_instance.is_running()


def libvirtd_stop():
    libvirtd_instance = Libvirtd()
    deprecation_warning()
    return libvirtd_instance.stop()


def libvirtd_restart():
    libvirtd_instance = Libvirtd()
    deprecation_warning()
    return libvirtd_instance.restart()


def service_libvirtd_control(action, session=None):
    libvirtd_instance = Libvirtd(session=session)
    deprecation_warning()
    getattr(libvirtd_instance, action)()


def unmark_storage_autostarted():
    """
    By removing this file libvirt start behavior at boot
    is simulated.
    """
    cmd = "rm -rf /var/run/libvirt/storage/autostarted"
    process.run(cmd, ignore_status=True, shell=True)



"""
Module simplifying manipulation of XML described at
http://libvirt.org/formatstorage.html#StoragePool
"""
import os
import logging
import tempfile

from avocado.utils import process

from .. import data_dir
from .. import libvirt_storage
from ..libvirt_xml import base, xcepts, accessors
from virttest import element_tree as ET


class SourceXML(base.LibvirtXMLBase):

    """
    Source block in pool xml, optionally containing different elements and
    attributes which dependent on pool type.
    """

    __slots__ = ('device_path', 'vg_name', 'host_name', 'dir_path',
                 'adp_type', 'adp_name', 'adp_parent', 'adp_wwnn',
                 'adp_wwpn', 'format_type', 'hosts', 'auth_type',
                 'auth_username', 'secret_usage', 'secret_uuid',
                 'iqn_name', 'protocol_ver')

    def __init__(self, virsh_instance=base.virsh):
        """
        Create new SourceXML instance.
        """
        accessors.XMLAttribute(property_name='device_path',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='device',
                               attribute='path')
        accessors.XMLElementText(property_name='vg_name',
                                 libvirtxml=self,
                                 parent_xpath='/',
                                 tag_name='name')
        accessors.XMLAttribute(property_name='host_name',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='host',
                               attribute='name')
        accessors.XMLAttribute(property_name='dir_path',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='dir',
                               attribute='path')
        accessors.XMLAttribute(property_name='adp_type',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='adapter',
                               attribute='type')
        accessors.XMLAttribute(property_name='adp_name',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='adapter',
                               attribute='name')
        accessors.XMLAttribute(property_name='adp_parent',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='adapter',
                               attribute='parent')
        accessors.XMLAttribute(property_name='adp_wwnn',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='adapter',
                               attribute='wwnn')
        accessors.XMLAttribute(property_name='adp_wwpn',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='adapter',
                               attribute='wwpn')
        accessors.XMLAttribute(property_name='format_type',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='format',
                               attribute='type')
        accessors.XMLElementList('hosts', self, parent_xpath='/',
                                 marshal_from=self.marshal_from_host,
                                 marshal_to=self.marshal_to_host)
        accessors.XMLAttribute(property_name='auth_type',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='auth',
                               attribute='type')
        accessors.XMLAttribute(property_name='auth_username',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='auth',
                               attribute='username')
        accessors.XMLAttribute(property_name='secret_usage',
                               libvirtxml=self,
                               parent_xpath='/auth',
                               tag_name='secret',
                               attribute='usage')
        accessors.XMLAttribute(property_name='secret_uuid',
                               libvirtxml=self,
                               parent_xpath='/auth',
                               tag_name='secret',
                               attribute='uuid')
        accessors.XMLAttribute(property_name='iqn_name',
                               libvirtxml=self,
                               parent_xpath='/initiator',
                               tag_name='iqn',
                               attribute='name')
        accessors.XMLAttribute(property_name='protocol_ver',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='protocol',
                               attribute='ver')

        super(SourceXML, self).__init__(virsh_instance=virsh_instance)
        self.xml = u"<source></source>"

    @staticmethod
    def marshal_from_host(item, index, libvirtxml):
        """Convert a dictionary into a tag + attributes"""
        del index           # not used
        del libvirtxml      # not used
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of host "
                                         "attributes, not a %s"
                                         % str(item))
        return ('host', dict(item))  # return copy of dict, not reference

    @staticmethod
    def marshal_to_host(tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into a dictionary"""
        del index                    # not used
        del libvirtxml               # not used
        if tag != 'host':
            return None              # skip this one
        return dict(attr_dict)       # return copy of dict, not reference


class PoolXMLBase(base.LibvirtXMLBase):

    """
    Accessor methods for PoolXML class.

    Properties:
        pool_type:
            string, pool type
        name:
            string, pool name
        uuid:
            string, pool uuid
        capacity:
            integer, pool total capacity
        allocation:
            integer, pool allocated capacity
        available:
            integer, pool available capacity
        source:
            PoolSourceXML instanc
        target:
            string, target path of pool
    """

    __slots__ = ('pool_type', 'name', 'uuid', 'capacity',
                 'allocation', 'available', 'source', 'target_path',
                 'mode', 'owner', 'group')
    __uncompareable__ = base.LibvirtXMLBase.__uncompareable__

    __schema_name__ = "pool"

    def __init__(self, virsh_instance=base.virsh):
        accessors.XMLAttribute(property_name='pool_type',
                               libvirtxml=self,
                               parent_xpath='/',
                               tag_name='pool',
                               attribute='type')
        accessors.XMLElementText(property_name='name',
                                 libvirtxml=self,
                                 parent_xpath='/',
                                 tag_name='name')
        accessors.XMLElementText(property_name='uuid',
                                 libvirtxml=self,
                                 parent_xpath='/',
                                 tag_name='uuid')
        accessors.XMLElementInt(property_name='capacity',
                                libvirtxml=self,
                                parent_xpath='/',
                                tag_name='capacity')
        accessors.XMLElementInt(property_name='allocation',
                                libvirtxml=self,
                                parent_xpath='/',
                                tag_name='allocation')
        accessors.XMLElementInt(property_name='available',
                                libvirtxml=self,
                                parent_xpath='/',
                                tag_name='available')
        accessors.XMLElementText(property_name='target_path',
                                 libvirtxml=self,
                                 parent_xpath='/target',
                                 tag_name='path')
        accessors.XMLElementText(property_name='mode',
                                 libvirtxml=self,
                                 parent_xpath='/target/permissions',
                                 tag_name='mode')
        accessors.XMLElementInt(property_name='owner',
                                libvirtxml=self,
                                parent_xpath='/target/permissions',
                                tag_name='owner')
        accessors.XMLElementInt(property_name='group',
                                libvirtxml=self,
                                parent_xpath='/target/permissions',
                                tag_name='group')
        super(PoolXMLBase, self).__init__(virsh_instance=virsh_instance)

    def get_source(self):
        xmltreefile = self.__dict_get__('xml')
        try:
            source_root = xmltreefile.reroot('/source')
        except KeyError as detail:
            raise xcepts.LibvirtXMLError(detail)
        sourcexml = SourceXML(virsh_instance=self.__dict_get__('virsh'))
        sourcexml.xmltreefile = source_root
        return sourcexml

    def del_source(self):
        xmltreefile = self.__dict_get__('xml')
        element = xmltreefile.find('/source')
        if element is not None:
            xmltreefile.remove(element)
            xmltreefile.write()

    def set_source(self, value):
        if not issubclass(type(value), SourceXML):
            raise xcepts.LibvirtXMLError(
                "Value must be a SourceXML or subclass")
        xmltreefile = self.__dict_get__('xml')
        self.del_source()
        root = xmltreefile.getroot()
        root.append(value.xmltreefile.getroot())
        xmltreefile.write()

    def add_source(self, tag, attr, text=None):
        xmltreefile = self.__dict_get__('xml')
        try:
            node = xmltreefile.find('/source')
        except KeyError as detail:
            raise xcepts.LibvirtXMLError(detail)
        if node is not None:
            ET.SubElement(node, tag, attr, text)
        xmltreefile.write()


class PoolcapabilityXML(base.LibvirtXMLBase):

    """
    Handler of libvirt pool-capabilities.

    """

    # TODO: Add more __slots__ and accessors to get some useful stats
    # e.g. guest_count etc.

    __schema_name__ = "poolcapabilities"

    def __init__(self, virsh_instance=base.virsh):
        super(PoolcapabilityXML, self).__init__(virsh_instance)
        # calls set_xml accessor method
        self['xml'] = self.__dict_get__('virsh').pool_capabilities()

    def get_pool_capabilities(self):
        """
        Accessor method for pool-type property (in __slots__).
        Return a pool info dict in following schema:
        {<pool_type>:{<pool_default_format>:[<pool_value>,...],
         <vol_default_format>:[<vol_value>,...]}}
        """
        pool_cal = {}
        pool_default_format_name = ''
        vol_default_format_name = ''
        poolxmltreefile = self.__dict_get__('xml')
        for pooltype in poolxmltreefile.findall('pool'):
            pool_type = pooltype.get('type')
            pool_type_info = pool_cal.get(pool_type, {})
            if pooltype.find('poolOptions') is not None:
                pool_options = pooltype.find('poolOptions')
                pool_enum = pool_options.find('enum')
                for default_pool_format in pool_options.findall('defaultFormat'):
                    pool_default_format_name = default_pool_format.get('type')
                pool_list = []
                for pool_value in pool_enum.findall('value'):
                    pool_value_name = pool_value.text
                    pool_list.append(pool_value_name)
                pool_type_info[pool_default_format_name] = pool_list
                if pooltype.find('volOptions') is not None:
                    vol_options = pooltype.find('volOptions')
                    enum_name = vol_options.find('enum')
                    for default_vol_format in vol_options.findall('defaultFormat'):
                        vol_default_format_name = default_vol_format.get('type')
                    vol_list = []
                    for vol_value in enum_name.findall('value'):
                        vol_value_name = vol_value.text
                        vol_list.append(vol_value_name)
                    pool_type_info[vol_default_format_name] = vol_list
                else:
                    pool_type_info['vol_default_format_name'] = []
                pool_cal[pool_type] = pool_type_info
            else:
                pool_type_info['pool_default_format_name'] = []
                vol_list = []
                if pooltype.find('volOptions') is not None:
                    vol_options = pooltype.find('volOptions')
                    enum_name = vol_options.find('enum')
                    for default_vol_format in vol_options.findall('defaultFormat'):
                        vol_default_format_name = default_vol_format.get('type')
                        if pool_type == 'rbd':
                            vol_list.append()
                        else:
                            for vol_value in enum_name.findall('value'):
                                vol_value_name = vol_value.text
                                vol_list.append(vol_value_name)
                            pool_type_info[vol_default_format_name] = vol_list
                else:
                    pool_type_info['vol_default_format_name'] = []
                pool_cal[pool_type] = pool_type_info
        return pool_cal


class PoolXML(PoolXMLBase):

    """
    Manipulators of a libvirt Pool through it's XML definition.
    """

    __slots__ = []

    def __init__(self, pool_type='dir', virsh_instance=base.virsh):
        """
        Initialize new instance with empty XML
        """
        super(PoolXML, self).__init__(virsh_instance=virsh_instance)
        self.xml = u"<pool type='%s'></pool>" % pool_type

    @staticmethod
    def new_from_dumpxml(name, virsh_instance=base.virsh):
        """
        Return new PoolXML instance from virsh pool-dumpxml command

        :param name: Name of pool to pool-dumpxml
        :param virsh_instance: Virsh module or instance to use
        :return: new initialized PoolXML instance
        """
        pool_xml = PoolXML(virsh_instance=virsh_instance)
        pool_xml['xml'] = virsh_instance.pool_dumpxml(name)
        return pool_xml

    @staticmethod
    def get_type(name, virsh_instance=base.virsh):
        """
        Return pool type by pool name

        :param name: pool name
        :return: pool type
        """
        pool_xml = PoolXML.new_from_dumpxml(name, virsh_instance)
        return pool_xml.pool_type

    @staticmethod
    def get_pool_details(name, virsh_instance=base.virsh):
        """
        Return pool details by pool name.

        :param name: pool name
        :return: a dict which include a series of pool details
        """
        pool_xml = PoolXML.new_from_dumpxml(name, virsh_instance)
        pool_details = {}
        pool_details['type'] = pool_xml.pool_type
        pool_details['uuid'] = pool_xml.uuid
        pool_details['capacity'] = pool_xml.capacity
        pool_details['allocation'] = pool_xml.allocation
        pool_details['available'] = pool_xml.available
        if pool_xml.pool_type != "gluster":
            pool_details['target_path'] = pool_xml.target_path
        return pool_details

    def pool_undefine(self):
        """
        Undefine pool with libvirt retaining XML in instance
        """
        try:
            self.virsh.pool_undefine(self.name, ignore_status=False)
        except process.CmdError:
            logging.error("Undefine pool '%s' failed.", self.name)
            return False

    def pool_define(self):
        """
        Define pool with virsh from this instance
        """
        result = self.virsh.pool_define(self.xml)
        if result.exit_status:
            logging.error("Define %s failed.\n"
                          "Detail: %s.", self.name,
                          result.stderr_text)
            return False
        return True

    @staticmethod
    def pool_rename(name, new_name, uuid=None, virsh_instance=base.virsh):
        """
        Rename a pool from pool XML.
        :param name: Original pool name.
        :param new_name: new name of pool.
        :param uuid: new pool uuid, if None libvirt will generate automatically.
        :return: True/False or raise LibvirtXMLError
        """
        pool_ins = libvirt_storage.StoragePool()
        if not pool_ins.is_pool_persistent(name):
            logging.error("Cannot rename for transient pool")
            return False
        start_pool = False
        if pool_ins.is_pool_active(name):
            start_pool = True
        poolxml = PoolXML.new_from_dumpxml(name, virsh_instance)
        backup = poolxml.copy()

        def _cleanup(details=""):
            # cleanup if rename failed
            backup.pool_define()
            if start_pool:
                pool_ins.start_pool(name)
            raise xcepts.LibvirtXMLError("%s" % details)

        if not pool_ins.delete_pool(name):
            _cleanup(details="Delete pool %s failed" % name)
        # Alter the XML
        poolxml.name = new_name
        if uuid is None:
            del poolxml.uuid
        else:
            poolxml.uuid = uuid
        # Re-define XML to libvirt
        logging.debug("Rename pool: %s to %s.", name, new_name)
        # error message for failed define
        error_msg = "Error reported while defining pool:\n"
        try:
            if not poolxml.pool_define():
                raise xcepts.LibvirtXMLError(error_msg + "%s"
                                             % poolxml.get('xml'))
        except process.CmdError as detail:
            del poolxml
            # Allow exceptions thrown here since state will be undefined
            backup.pool_define()
            raise xcepts.LibvirtXMLError(error_msg + "%s" % detail)
        if not poolxml.pool_define():
            logging.info("Pool xml: %s" % poolxml.get('xml'))
            _cleanup(details="Define pool %s failed" % new_name)
        if start_pool:
            pool_ins.start_pool(new_name)
        return True

    @staticmethod
    def backup_xml(name, virsh_instance=base.virsh):
        """
        Backup the pool xml file.
        """
        try:
            xml_file = tempfile.mktemp(dir=data_dir.get_tmp_dir())
            virsh_instance.pool_dumpxml(name, to_file=xml_file)
            return xml_file
        except Exception as detail:
            if os.path.exists(xml_file):
                os.remove(xml_file)
            logging.error("Failed to backup xml file:\n%s", detail)
            return ""

    def debug_xml(self):
        """
        Dump contents of XML file for debugging
        """
        xml = str(self)
        for debug_line in str(xml).splitlines():
            logging.debug("Pool XML: %s", debug_line)



#!/usr/bin/python

import os
import socket
import struct
import optparse

try:
    import hashlib
except ImportError:
    import md5


class Md5MissMatch(Exception):

    def __init__(self, md5_pre, md5_post):
        Exception.__init__(self, md5_pre, md5_post)
        self.md5_pre = md5_pre
        self.md5_post = md5_post

    def __str__(self):
        return ("Md5 miss match. Original md5 = %s, current md5 = %s" %
                (self.md5_pre, self.md5_post))


class ShakeHandError(Exception):

    def __init__(self, msg):
        Exception.__init__(self, msg)
        self.msg = msg

    def __str__(self):
        return ("Shake hand fail. %s" % self.msg)


def md5_init(data=None):
    """
    Returns md5. This function is implemented in order to encapsulate hash
    objects in a way that is compatible with python 2.4 and python 2.6
    without warnings.

    Note that even though python 2.6 hashlib supports hash types other than
    md5 and sha1, we are artificially limiting the input values in order to
    make the function to behave exactly the same among both python
    implementations.

    :param data: Optional input string that will be used to update the hash.
    """

    try:
        md5_value = hashlib.new("md5")
    except NameError:
        md5_value = md5.new()
    if data:
        md5_value.update(data)
    return md5_value


def get_md5(filename, size=None):
    """
    Calculate the hash of filename.
    If size is not None, limit to first size bytes.
    Throw exception if something is wrong with filename.
    Can be also implemented with bash one-liner (assuming size%1024==0):
    dd if=filename bs=1024 count=size/1024 | sha1sum -

    :param filename: Path of the file that will have its hash calculated.
    :param method: Method used to calculate the hash. Supported methods:
            * md5
            * sha1
    :return: Hash of the file, if something goes wrong, return None.
    """
    chunksize = 4096
    fsize = os.path.getsize(filename)

    if not size or size > fsize:
        size = fsize
    f = open(filename, 'rb')

    md5_value = md5_init()
    while size > 0:
        if chunksize > size:
            chunksize = size
        data = f.read(chunksize)
        if len(data) == 0:
            print("Nothing left to read but size=%d" % size)
            break
        md5_value.update(data)
        size -= len(data)
    f.close()
    return md5_value.hexdigest()


def read(connect, exp_len, connect_type):
    if connect_type == 'socket':
        return connect.recv(exp_len)
    else:
        return os.read(connect, exp_len)


def write(connect, txt, connect_type):
    if connect_type == 'socket':
        connect.send(txt)
    else:
        os.write(connect, txt)


def shake_hand(connect, size=0, action="receive", connect_type='socket'):
    hi_str = struct.pack("2s", b"HI")
    hi_str_len = len(hi_str)
    if action == "send":
        write(connect, hi_str, connect_type)
        txt = read(connect, hi_str_len, connect_type)
        hi_str = struct.unpack("2s", txt)[0]
        if hi_str != b"HI":
            raise ShakeHandError("Fail to get HI from guest.")
        size_str = struct.pack("q", size)
        write(connect, size_str, connect_type)
        txt = read(connect, 3, connect_type)
        ack_str = struct.unpack("3s", txt)[0]
        if ack_str != b"ACK":
            raise ShakeHandError("Guest did not ACK the file size message.")
        return size
    elif action == "receive":
        txt = b''
        while len(txt) < hi_str_len:
            txt += read(connect, hi_str_len, connect_type)
        hi_str = struct.unpack("2s", txt)[0]
        if hi_str != b"HI":
            raise ShakeHandError("Fail to get HI from guest.")
        write(connect, hi_str, connect_type)
        size = read(connect, 8, connect_type)
        if size:
            size = struct.unpack("q", size)[0]
            txt = struct.pack("3s", b"ACK")
            write(connect, txt, connect_type)
        return size


def receive(connect, filename, p_size=1024, connect_type='socket'):
    recv_size = 0
    size = shake_hand(connect, action="receive", connect_type=connect_type)
    if p_size < int(size):
        p_size = int(size)
    md5_value = md5_init()
    file_no = open(filename, 'wb')
    try:
        while recv_size < size:
            txt = read(connect, p_size, connect_type)
            file_no.write(txt)
            md5_value.update(txt)
            recv_size += len(txt)
    finally:
        file_no.close()
    md5_sum = md5_value.hexdigest()
    return md5_sum


def send(connect, filename, p_size=1024, connect_type='socket'):
    send_size = 0
    f_size = os.path.getsize(filename)
    shake_hand(connect, f_size, action="send", connect_type=connect_type)
    md5_value = md5_init()
    file_no = open(filename, 'rb')
    try:
        while send_size < f_size:
            txt = file_no.read(p_size)
            write(connect, txt, connect_type)
            md5_value.update(txt)
            send_size += len(txt)
    finally:
        print("Sent size = %s" % send_size)
        file_no.close()
    md5_sum = md5_value.hexdigest()
    wait_receive_ack(connect, connect_type)
    return md5_sum


def wait_receive_ack(connect, connect_type):
    exp_str = b'ALLRECEIVED'
    exp_size = len(exp_str)
    r_str = b''
    r_size = 0
    while r_size < exp_size:
        txt = read(connect, exp_size, connect_type)
        r_size += len(txt)
        r_str += txt
    if struct.unpack("11s", txt)[0] != exp_str:
        raise ShakeHandError("Not get Received ACK from guest.")


def close_connect(connect, connect_type):
    if connect_type == 'socket':
        connect.shutdown(socket.SHUT_RDWR)
        connect.close()
    else:
        os.close(connect)


def main():
    parser = optparse.OptionParser("Transfer data between guest and host"
                                   "through virtio serial. Please make sure"
                                   "VirtIOChannel.py run in guest first.")
    parser.add_option("-t", "--type", dest="type", default="unix_socket",
                      help="The device type of chardev, eg:unix_socket,"
                           " tcp_socket, udp, etc. unix_socket by default")
    parser.add_option("-s", "--socket", dest="socket",
                      help="The host device used in qemu command"
                           "eg:your CLI:-chardev socket,id=channel2,"
                           "path=/tmp/helloworld2 ,then input"
                           "'/tmp/helloworld2' here. Or '127.0.0.1:6001'"
                           " for tcp/udp socket")
    parser.add_option("-f", "--filename", dest="filename",
                      help="File transfer to guest or save data to.")
    parser.add_option("-a", "--action", dest="action", default="send",
                      help="Send data out or receive data.")
    parser.add_option("-p", "--package", dest="package", default=1024,
                      help="Package size during file transfer.")

    options, args = parser.parse_args()

    chardev_type = options.type
    if options.socket:
        device = options.socket
    else:
        parser.error("Please set -s parameter.")

    if options.filename:
        filename = options.filename
    else:
        parser.error("Please set -f parameter.")
    action = options.action
    p_size = options.package
    if action not in ("receive", "send", "both"):
        parser.error('Please set -a parameter: "receive", "send", "both"')

    device_property = {'pty': {'connect_type': 'device'},
                       'unix_socket': {'connect_type': 'socket',
                                       'sock_flag': socket.AF_UNIX,
                                       'sock_type': socket.SOCK_STREAM},
                       'tcp_socket': {'connect_type': 'socket',
                                      'sock_flag': socket.AF_INET,
                                      'sock_type': socket.SOCK_STREAM},
                       'udp': {'connect_type': 'socket',
                               'sock_flag': socket.AF_INET,
                               'sock_type': socket.SOCK_DGRAM}}
    if chardev_type in ('tcp_socket', 'udp'):
        device = device.split(':')
        device = (device[0], int(device[1]))
    prop = device_property.get(chardev_type)
    connect_type = prop.get('connect_type')
    if connect_type == 'socket':
        vport = socket.socket(prop.get('sock_flag'),
                              prop.get('sock_type'))
        vport.connect(device)
    else:
        vport = os.open(device, os.O_RDWR)
    if action == "receive":
        md5_sum = receive(vport, filename, p_size=p_size,
                          connect_type=connect_type)
        print("md5_sum = %s" % md5_sum)
    elif action == "send":
        md5_sum = send(vport, filename, p_size=p_size,
                       connect_type=connect_type)
        print("md5_sum = %s" % md5_sum)
    else:
        md5_ori = send(vport, filename, p_size=p_size,
                       connect_type=connect_type)
        print("md5_original = %s" % md5_ori)
        md5_post = receive(vport, filename, p_size=p_size,
                           connect_type=connect_type)
        print("md5_post = %s" % md5_post)
        if md5_ori != md5_post:
            raise Md5MissMatch(md5_ori, md5_post)
    close_connect(vport, connect_type)


if __name__ == "__main__":
    main()



"""
Interfaces and helpers for the virtio_serial ports.

:copyright: 2012 Red Hat Inc.
"""
from __future__ import division
from threading import Thread
from collections import deque
import logging
import os
import random
import select
import socket
import time
import struct

import aexpect

from avocado.core import exceptions
from avocado.utils import process

from six.moves import xrange

from virttest import data_dir

SOCKET_SIZE = 2048


class VirtioPortException(Exception):

    """ General virtio_port exception """
    pass


class VirtioPortFatalException(VirtioPortException):

    """ Fatal virtio_port exception """
    pass


class _VirtioPort(object):

    """
    Define structure to keep information about used port.
    """

    def __init__(self, qemu_id, name, hostfile, port_type="unix_socket"):
        """
        :param name: Name of port for guest side.
        :param hostfile: Path to port on host side.
        """
        self.qemu_id = qemu_id
        self.name = name
        self.hostfile = hostfile
        self.is_console = None  # "yes", "no"
        self.sock = None
        self.port_was_opened = None
        self.port_type = port_type

    def __str__(self):
        """
        Convert to text.
        """
        return ("%s,%s,%s,%s,%d" % ("Socket", self.name, self.is_console,
                                    self.hostfile, self.is_open()))

    def __getstate__(self):
        """
        socket is unpickable so we need to remove it and say it's closed.
        Used by autotest env.
        """
        # TODO: add port cleanup into qemu_vm.py
        if self.is_open():
            logging.warn("Force closing virtio_port socket, FIX the code to "
                         " close the socket prior this to avoid possible err.")
            self.close()
        return self.__dict__.copy()

    def is_open(self):
        """ :return: host port status (open/closed) """
        if self.sock:
            return True
        else:
            return False

    def for_guest(self):
        """
        Format data for communication with guest side.
        """
        return [self.name, self.is_console]

    def open(self):     # @ReservedAssignment
        """
        Open port on host side.
        """
        if self.is_open():
            return
        attempt = 11
        while attempt > 0:
            try:
                if self.port_type == 'unix_socket':
                    sock_flag = socket.AF_UNIX
                elif self.port_type in ('tcp_socket', 'udp'):
                    sock_flag = socket.AF_INET
                if self.port_type == 'udp':
                    sock_type = socket.SOCK_DGRAM
                elif self.port_type in ('tcp_socket', 'unix_socket'):
                    sock_type = socket.SOCK_STREAM
                self.sock = socket.socket(sock_flag, sock_type)
                self.sock.settimeout(1)
                self.sock.connect(self.hostfile)
                self.sock.setsockopt(1, socket.SO_SNDBUF, SOCKET_SIZE)
                self.sock.settimeout(None)
                self.port_was_opened = True
                return
            except Exception:
                attempt -= 1
                time.sleep(1)
        raise exceptions.TestFail("Can't open the %s sock (%s)" % (self.name,
                                                                   self.hostfile))

    def clean_port(self):
        """
        Clean all data from opened port on host side.
        """
        if self.is_open():
            self.close()
        elif not self.port_was_opened:
            # BUG: Don't even try opening port which was never used. It
            # hangs for ever... (virtio_console bug)
            logging.debug("No need to clean port %s", self)
            return
        logging.debug("Cleaning port %s", self)
        self.open()
        ret = select.select([self.sock], [], [], 1.0)
        if ret[0]:
            buf = self.sock.recv(1024)
            logging.debug("Rest in socket: " + repr(buf))

    def close(self):
        """
        Close port.
        """
        if self.is_open():
            self.sock.shutdown(socket.SHUT_RDWR)
            self.sock.close()
            self.sock = None

    def mark_as_clean(self):
        """
        Mark port as cleaned
        """
        self.port_was_opened = False


class VirtioSerial(_VirtioPort):

    """ Class for handling virtio-serialport """

    def __init__(self, qemu_id, name, hostfile, port_type="unix_socket"):
        """
        :param name: Name of port for guest side.
        :param hostfile: Path to port on host side.
        """
        super(VirtioSerial, self).__init__(qemu_id, name, hostfile, port_type)
        self.is_console = "no"


class VirtioConsole(_VirtioPort):

    """ Class for handling virtio-console """

    def __init__(self, qemu_id, name, hostfile, port_type="unix_socket"):
        """
        :param name: Name of port for guest side.
        :param hostfile: Path to port on host side.
        """
        super(VirtioConsole, self).__init__(qemu_id, name, hostfile, port_type)
        self.is_console = "yes"


class GuestWorker(object):

    """
    Class for executing "virtio_console_guest" script on guest
    """

    def __init__(self, vm):
        """ Initialize worker for use (including port init on guest) """
        self.vm = vm
        self.session = self.vm.wait_for_login()
        self.__cmd_execute_worker = None

        # Detect the OS version
        guest_script_py = "virtio_console_guest.py"
        out = self.session.cmd_output("echo on")
        if "on" in out:
            self.os_linux = True
            guest_script_path = os.path.join("/tmp", guest_script_py)
            cmd_guest_size = ("du -b %s | cut -f1"
                              % guest_script_path)
            cmd_already_compiled_chck = "ls %so" % guest_script_path
            cmd_compile = ("`command -v python python3 | head -1` -OO %s -c "
                           "&& echo -n 'PASS: Compile virtio_guest finished' "
                           "|| echo -n 'FAIL: Compile virtio_guest failed'"
                           % guest_script_path)
            self.__cmd_execute_worker = ("`command -v python python3 | head -1` %so"
                                         "&& echo -n 'PASS: virtio_guest finished' "
                                         "|| echo -n 'FAIL: virtio_guest failed'"
                                         % guest_script_path)
        else:
            self.os_linux = False
            guest_script_path = "C:\\%s" % guest_script_py
            cmd_guest_size = ("for %%I in (%s) do @echo %%~zI"
                              % guest_script_path)
            cmd_already_compiled_chck = "dir %so" % guest_script_path
            cmd_compile = ("%s -c "
                           "&& echo PASS: Compile virtio_guest finished "
                           "|| echo FAIL: Compile virtio_guest failed"
                           % guest_script_path)
            self.__cmd_execute_worker = ("%so "
                                         "&& echo PASS: virtio_guest finished "
                                         "|| echo FAIL: virtio_guest failed"
                                         % guest_script_path)

        # Copy, compile and run the worker
        timeout = 120
        guest_script_src = os.path.join(data_dir.get_shared_dir(), 'scripts',
                                        'virtio_console_guest.py')
        script_size = process.run("du -b %s | cut -f1" % guest_script_src,
                                  shell=True).stdout_text.strip()
        script_size_guest = self.session.cmd_output(cmd_guest_size).strip()
        if (script_size != script_size_guest or
                self.session.cmd_status(cmd_already_compiled_chck)):
            if self.os_linux:
                # Disable serial-getty@hvc0.service on systemd-like hosts
                self.session.cmd_status('systemctl mask '
                                        'serial-getty@hvc0.service')
                self.session.cmd_status('systemctl stop '
                                        'serial-getty@hvc0.service')
            # Copy virtio_console_guest.py into guests
            self.vm.copy_files_to(guest_script_src, guest_script_path)

            # set echo off (self.cmd() musn't contain C:)
            self.session.sendline("echo off")
            # Compile worker
            logging.debug("Compile %s on guest %s", guest_script_py,
                          self.vm.name)
            try:
                self.cmd(cmd_compile, timeout)
            except VirtioPortException:
                if not self.os_linux:
                    logging.error("Script execution failed, do you have python"
                                  " and pywin32 installed? Currently this "
                                  "needs to be done manually!")
                raise
            self.session.sendline()

        # set echo off (self.cmd() musn't contain C:)
        self.session.sendline("echo off")
        logging.debug("Starting %so on guest %s", guest_script_py,
                      self.vm.name)
        self._execute_worker(timeout)
        self._init_guest(timeout)

    def _execute_worker(self, timeout=10):
        """ Execute worker on guest """
        try:
            self.cmd(self.__cmd_execute_worker, timeout)
        except VirtioPortException:
            if not self.os_linux:
                logging.error("Script execution failed, do you have python"
                              " and pywin32 installed? Currently this "
                              "needs to be done manually!")
            raise
        # Let the system rest
        # FIXME: Is this always necessarily?
        time.sleep(2)

    def _init_guest(self, timeout=10):
        """ Initialize worker on guest """
        ports = []
        for port in self.vm.virtio_ports:
            ports.append(port.for_guest())
        self.cmd("virt.init(%s)" % (ports), timeout)

    def reconnect(self, vm, timeout=10):
        """
        Reconnect to guest_worker (eg. after migration)
        :param vm: New VM object
        """
        self.vm = vm
        self.session = self.vm.wait_for_login()
        self._execute_worker(timeout)

    def cmd(self, cmd, timeout=10, patterns=None):
        """
        Wrapper around the self.cmd command which executes the command on
        guest. Unlike self._cmd command when the command fails it raises the
        test exceptions.
        :param command: Command that will be executed.
        :param timeout: Timeout used to verify expected output.
        :return: Tuple (match index, data)
        """
        match, data = self._cmd(cmd, timeout, patterns)
        if match == 1 or match is None:
            raise VirtioPortException("Failed to execute '%s' on"
                                      " virtio_console_guest.py, "
                                      "vm: %s, output:\n%s" %
                                      (cmd, self.vm.name, data))
        return (match, data)

    def _cmd(self, cmd, timeout=10, patterns=None):
        """
        Execute given command inside the script's main loop.
        :param command: Command that will be executed.
        :param timeout: Timeout used to verify expected output.
        :param patterns: Expected patterns; have to startwith ^PASS: or ^FAIL:
        :return: Tuple (match index, data)
        """
        if not patterns:
            patterns = ("^PASS:", "^FAIL:")
        logging.debug("Executing '%s' on virtio_console_guest.py,"
                      " vm: %s, timeout: %s", cmd, self.vm.name, timeout)
        self.session.sendline(cmd)
        try:
            (match, data) = self.session.read_until_any_line_matches(patterns,
                                                                     timeout=timeout)
            if patterns[match].startswith('^PASS:'):
                match = 0
            elif patterns[match].startswith('^FAIL:'):
                match = 1
            else:
                data = ("Incorrect pattern %s. Data in console:\n%s"
                        % (patterns[match], data))
                match = None
        except aexpect.ExpectError as inst:
            match = None
            data = "Cmd process timeout. Data in console:\n" + inst.output

        self.vm.verify_kernel_crash()

        return (match, data)

    def read_nonblocking(self, internal_timeout=None, timeout=None):
        """
        Reads-out all remaining output from GuestWorker.

        :param internal_timeout: Time (seconds) to wait before we give up
                                 reading from the child process, or None to
                                 use the default value.
        :param timeout: Timeout for reading child process output.
        """
        return self.session.read_nonblocking(internal_timeout, timeout)

    def _cleanup_ports(self):
        """
        Read all data from all ports, in both sides of each port.
        """
        for port in self.vm.virtio_ports:
            openned = port.is_open()
            port.clean_port()
            self.cmd("virt.clean_port('%s'),1024" % port.name, 10)
            if not openned:
                port.close()
                self.cmd("virt.close('%s'),1024" % port.name, 10)

    def safe_exit_loopback_threads(self, send_pts, recv_pts):
        """
        Safely executes on_guest("virt.exit_threads()") using workaround of
        the stuck thread in loopback in mode=virt.LOOP_NONE .
        :param send_pts: list of possible send sockets we need to work around.
        :param recv_pts: list of possible recv sockets we need to read-out.
        """
        # No need to clean ports when VM is dead
        if not self.vm or self.vm.is_dead():
            return
        # in LOOP_NONE mode it might stuck in read/write
        # This command can't fail, can only freze so wait for the correct msg
        match, tmp = self._cmd("virt.exit_threads()", 3, ("^PASS: All threads"
                                                          " finished",))
        if match is None:
            logging.warn("Workaround the stuck thread on guest")
            # Thread is stuck in read/write
            for send_pt in send_pts:
                timeout = None
                try:
                    timeout = send_pt.sock.gettimeout()
                    send_pt.sock.settimeout(1)
                    send_pt.sock.send(b".")
                except socket.timeout:
                    pass    # If still stuck VM gets destroyed below
                send_pt.sock.settimeout(timeout)
        elif match != 0:
            # Something else
            raise VirtioPortException("Unexpected fail\nMatch: %s\nData:\n%s"
                                      % (match, tmp))

        # Read-out all remaining data
        for recv_pt in recv_pts:
            while select.select([recv_pt.sock], [], [], 0.1)[0]:
                recv_pt.sock.recv(1024)

        # This will cause fail in case anything went wrong.
        match, tmp = self._cmd("print('PASS: nothing')", 10, ('^PASS: nothing',
                                                              '^FAIL:'))
        if match != 0:
            logging.error("Python is stuck/FAILed after read-out:\n%s", tmp)
            try:
                self.session.close()
                self.session = self.vm.wait_for_login()
                if self.os_linux:   # On windows it dies with the connection
                    self.cmd("killall -9 `command -v python python3 | head -1` "
                             "&& echo -n PASS: python killed"
                             "|| echo -n PASS: python was already dead", 10)
                self._execute_worker()
                self._init_guest()
            except Exception as inst:
                logging.error(inst)
                raise VirtioPortFatalException("virtio-console driver is "
                                               "irreparably blocked, further tests might FAIL.")

    def cleanup_ports(self):
        """
        Clean state of all ports and set port to default state.

        Default state: No data on port or in port buffer. Read mode = blocking.
        """
        # Check if python is still alive
        match, tmp = self._cmd("is_alive()", 10)
        if match != 0:
            logging.error("Python died/is stuck/have remaining threads")
            logging.debug(tmp)
            try:
                self.vm.verify_kernel_crash()

                match, tmp = self._cmd("guest_exit()", 10, ('^FAIL:',
                                                            '^PASS: virtio_guest finished'))
                self.session.close()
                self.session = self.vm.wait_for_login()
                # On windows it dies with the connection
                if match != 0 and self.os_linux:
                    logging.debug(tmp)
                    self.cmd("killall -9 `command -v python python3 | head -1` "
                             "&& echo -n PASS: python killed"
                             "|| echo -n PASS: python was already dead", 10)

                self._execute_worker()
                self._init_guest()
                self._cleanup_ports()

            except Exception as inst:
                logging.error(inst)
                raise VirtioPortFatalException("virtio-console driver is "
                                               "irreparably blocked, further tests might FAIL.")

    def cleanup(self):
        """ Cleanup ports and quit the worker """
        # Verify that guest works
        if self.session and self.vm and self.vm.is_alive():
            self.cleanup_ports()
        if self.vm:
            self.vm.verify_kernel_crash()
        # Quit worker
        if self.session and self.vm and self.vm.is_alive():
            match, tmp = self._cmd("guest_exit()", 10)
            self.session.close()
            # On windows it dies with the connection
            if match != 0 and self.os_linux:
                logging.warn('guest_worker stuck during cleanup:\n%s\n,'
                             ' killing python...', tmp)
                self.session = self.vm.wait_for_login()
                self.cmd("killall -9 `command -v python python3 | head -1` "
                         "&& echo -n PASS: python killed"
                         "|| echo -n PASS: python was already dead", 10)
                self.session.close()
        self.session = None
        self.vm = None


class ThSend(Thread):

    """
    Random data sender thread.
    """

    def __init__(self, port, data, exit_event, quiet=False):
        """
        :param port: Destination port.
        :param data: The data intend to be send in a loop.
        :param exit_event: Exit event.
        :param quiet: If true don't raise event when crash.
        """
        Thread.__init__(self)
        self.port = port
        # FIXME: socket.send(data>>127998) without read blocks thread
        if len(data) > 102400:
            data = data[0:102400]
            logging.error("Data is too long, using only first %d bytes",
                          len(data))
        self.data = data
        self.exitevent = exit_event
        self.idx = 0
        self.quiet = quiet
        self.ret_code = 1    # sets to 0 when finish properly

    def run(self):
        logging.debug("ThSend %s: run", self.getName())
        try:
            while not self.exitevent.isSet():
                self.idx += self.port.send(self.data)
            logging.debug("ThSend %s: exit(%d)", self.getName(),
                          self.idx)
        except Exception as ints:
            if not self.quiet:
                raise ints
            logging.debug(ints)
        self.ret_code = 0


class ThSendCheck(Thread):

    """
    Random data sender thread.
    """

    def __init__(self, port, exit_event, queues, blocklen=1024,
                 migrate_event=None, reduced_set=False):
        """
        :param port: Destination port
        :param exit_event: Exit event
        :param queues: Queues for the control data (FIFOs)
        :param blocklen: Block length
        :param migrate_event: Event indicating port was changed and is ready.
        """
        Thread.__init__(self)
        self.port = port
        self.port.sock.settimeout(1)
        self.queues = queues
        # FIXME: socket.send(data>>127998) without read blocks thread
        if blocklen > 102400:
            blocklen = 102400
            logging.error("Data is too long, using blocklen = %d",
                          blocklen)
        self.blocklen = blocklen
        self.exitevent = exit_event
        self.migrate_event = migrate_event
        self.idx = 0
        self.ret_code = 1    # sets to 0 when finish properly
        self.reduced_set = reduced_set

    def run(self):
        logging.debug("ThSendCheck %s: run", self.getName())
        _err_msg_exception = ('ThSendCheck ' + str(self.getName()) + ': Got '
                              'exception %s, continuing')
        _err_msg_disconnect = ('ThSendCheck ' + str(self.getName()) + ': Port '
                               'disconnected, waiting for new port.')
        _err_msg_reconnect = ('ThSendCheck ' + str(self.getName()) + ': Port '
                              'reconnected, continuing.')
        too_much_data = False
        if self.reduced_set:
            rand_a = 65
            rand_b = 91
        else:
            rand_a = 0
            rand_b = 255
        while not self.exitevent.isSet():
            # FIXME: workaround the problem with qemu-kvm stall when too
            # much data is sent without receiving
            for queue in self.queues:
                while not self.exitevent.isSet() and len(queue) > 1048576:
                    too_much_data = True
                    time.sleep(0.1)
            try:
                ret = select.select([], [self.port.sock], [], 1.0)
            except Exception as inst:
                # self.port is not yet set while reconnecting
                if self.migrate_event is None:
                    raise exceptions.TestFail("ThSendCheck %s: Broken pipe. If this"
                                              " is expected behavior set migrate_event "
                                              "to support reconnection." % self.getName())
                if self.port.sock is None:
                    logging.debug(_err_msg_disconnect)
                    while self.port.sock is None:
                        if self.exitevent.isSet():
                            break
                        time.sleep(0.1)
                    logging.debug(_err_msg_reconnect)
                else:
                    logging.debug(_err_msg_exception, inst)
                continue
            if ret[1]:
                # Generate blocklen of random data add them to the FIFO
                # and send them over virtio_console
                buf = b""
                for _ in range(self.blocklen):
                    char = b"%c" % random.randrange(rand_a, rand_b)
                    buf += char
                    for queue in self.queues:
                        queue.append(char)
                target = self.idx + self.blocklen
                while not self.exitevent.isSet() and self.idx < target:
                    try:
                        idx = self.port.sock.send(buf)
                    except socket.timeout:
                        continue
                    except Exception as inst:
                        # Broken pipe
                        if not hasattr(inst, 'errno') or inst.errno != 32:
                            continue
                        if self.migrate_event is None:
                            self.exitevent.set()
                            raise exceptions.TestFail("ThSendCheck %s: Broken "
                                                      "pipe. If this is expected behavior "
                                                      "set migrate_event to support "
                                                      "reconnection." % self.getName())
                        logging.debug("ThSendCheck %s: Broken pipe "
                                      ", reconnecting. ", self.getName())
                        attempt = 10
                        while (attempt > 1 and
                               not self.exitevent.isSet()):
                            # Wait until main thread sets the new self.port
                            while not (self.exitevent.isSet() or
                                       self.migrate_event.wait(1)):
                                pass
                            if self.exitevent.isSet():
                                break
                            logging.debug("ThSendCheck %s: Broken pipe resumed"
                                          ", reconnecting...", self.getName())
                            self.port.sock = False
                            self.port.open()
                            try:
                                idx = self.port.sock.send(buf)
                            except Exception:
                                attempt -= 1
                                time.sleep(10)
                            else:
                                attempt = 0
                    buf = buf[idx:]
                    self.idx += idx
        logging.debug("ThSendCheck %s: exit(%d)", self.getName(),
                      self.idx)
        if too_much_data:
            logging.error("ThSendCheck: working around the 'too_much_data'"
                          "bug")
        self.ret_code = 0


class ThRecv(Thread):

    """
    Receives data and throws it away.
    """

    def __init__(self, port, event, blocklen=1024, quiet=False):
        """
        :param port: Data source port.
        :param event: Exit event.
        :param blocklen: Block length.
        :param quiet: If true don't raise event when crash.
        """
        Thread.__init__(self)
        self.port = port
        self._port_timeout = self.port.gettimeout()
        self.port.settimeout(0.1)
        self.exitevent = event
        self.blocklen = blocklen
        self.idx = 0
        self.quiet = quiet
        self.ret_code = 1    # sets to 0 when finish properly

    def run(self):
        logging.debug("ThRecv %s: run", self.getName())
        try:
            while not self.exitevent.isSet():
                # TODO: Workaround, it didn't work with select :-/
                try:
                    self.idx += len(self.port.recv(self.blocklen))
                except socket.timeout:
                    pass
            self.port.settimeout(self._port_timeout)
            logging.debug("ThRecv %s: exit(%d)", self.getName(), self.idx)
        except Exception as ints:
            if not self.quiet:
                raise ints
            logging.debug(ints)
        self.ret_code = 0


class ThRecvCheck(Thread):

    """
    Random data receiver/checker thread.
    """

    def __init__(self, port, buff, exit_event, blocklen=1024, sendlen=0,
                 migrate_event=None, debug=None):
        """
        :param port: Source port.
        :param buff: Control data buffer (FIFO).
        :param exit_event: Exit event.
        :param blocklen: Block length.
        :param sendlen: Block length of the send function (on guest)
        :param migrate_event: Event indicating port was changed and is ready.
        :param debug: Set the execution mode, when nothing run normal.
        """
        Thread.__init__(self)
        self.port = port
        self.buff = buff
        self.exitevent = exit_event
        self.migrate_event = migrate_event
        self.blocklen = blocklen
        self.idx = 0
        self.sendlen = sendlen + 1  # >=
        self.ret_code = 1    # sets to 0 when finish properly
        self.debug = debug      # see the self.run_* docstrings for details
        # self.sendidx is the maxiaml number of skipped/duplicated values
        # 1) autoreload when the host socket is reconnected. In this case
        #    it waits <30s for migrate_event and reloads sendidx to sendlen
        # 2) manual write to this value (eg. before you reconnect guest port).
        #    RecvThread decreases this value whenever data loss/dup occurs.
        self.sendidx = -1
        self.minsendidx = self.sendlen

    def reload_loss_idx(self):
        """
        This function reloads the acceptable loss to the original value
        (Reload the self.sendidx to self.sendlen)
        :note: This function is automatically called during port reconnection.
        """
        if self.sendidx >= 0:
            self.minsendidx = min(self.minsendidx, self.sendidx)
            logging.debug("ThRecvCheck %s: Previous data loss was %d.",
                          self.getName(), (self.sendlen - self.sendidx))
        self.sendidx = self.sendlen

    def run(self):
        """ Pick the right mode and execute it """
        if self.debug == 'debug':
            self.run_debug()
        elif self.debug == 'normal' or not self.debug:
            self.run_normal()
        else:
            logging.error('ThRecvCheck %s: Unsupported debug mode, using '
                          'normal mode.', self.getName())
            self.run_normal()

    def run_normal(self):
        """
        Receives data and verifies, whether they match the self.buff (queue).
        It allow data loss up to self.sendidx which can be manually loaded
        after host socket reconnection or you can overwrite this value from
        other thread.
        """
        logging.debug("ThRecvCheck %s: run", self.getName())
        _err_msg_missing_migrate_ev = ("ThRecvCheck %s: Broken pipe. If "
                                       "this is expected behavior set migrate_event to "
                                       "support reconnection." % self.getName())
        _err_msg_exception = ('ThRecvCheck ' + str(self.getName()) + ': Got '
                              'exception %s, continuing')
        _err_msg_disconnect = ('ThRecvCheck ' + str(self.getName()) + ': Port '
                               'disconnected, waiting for new port.')
        _err_msg_reconnect = ('ThRecvCheck ' + str(self.getName()) + ': Port '
                              'reconnected, continuing.')
        attempt = 10
        while not self.exitevent.isSet():
            try:
                ret = select.select([self.port.sock], [], [], 1.0)
            except Exception as inst:
                # self.port is not yet set while reconnecting
                if self.port.sock is None:
                    logging.debug(_err_msg_disconnect)
                    while self.port.sock is None:
                        if self.exitevent.isSet():
                            break
                        time.sleep(0.1)
                    logging.debug(_err_msg_reconnect)
                else:
                    logging.debug(_err_msg_exception, inst)
                continue
            if ret[0] and (not self.exitevent.isSet()):
                try:
                    buf = self.port.sock.recv(self.blocklen)
                except Exception as inst:
                    # self.port is not yet set while reconnecting
                    if self.port.sock is None:
                        logging.debug(_err_msg_disconnect)
                        while self.port.sock is None:
                            if self.exitevent.isSet():
                                break
                            time.sleep(0.1)
                        logging.debug(_err_msg_reconnect)
                    else:
                        logging.debug(_err_msg_exception, inst)
                    continue
                if buf:
                    # Compare the received data with the control data
                    for char in bytearray(buf):
                        char = struct.pack('B', char)
                        _char = self.buff.popleft()
                        if char == _char:
                            self.idx += 1
                        else:
                            # TODO BUG: data from the socket on host can
                            # be lost during migration
                            while char != _char:
                                if self.sendidx > 0:
                                    self.sendidx -= 1
                                    _char = self.buff.popleft()
                                else:
                                    self.exitevent.set()
                                    logging.error("ThRecvCheck %s: "
                                                  "Failed to recv %dth "
                                                  "character",
                                                  self.getName(), self.idx)
                                    logging.error("ThRecvCheck %s: "
                                                  "%s != %s",
                                                  self.getName(),
                                                  repr(char), repr(_char))
                                    logging.error("ThRecvCheck %s: "
                                                  "Recv = %s",
                                                  self.getName(), repr(buf))
                                    # sender might change the buff :-(
                                    time.sleep(1)
                                    _char = b""
                                    for buf in self.buff:
                                        _char += buf
                                        _char += b' '
                                    logging.error("ThRecvCheck %s: "
                                                  "Queue = %s",
                                                  self.getName(), repr(_char))
                                    logging.info("ThRecvCheck %s: "
                                                 "MaxSendIDX = %d",
                                                 self.getName(),
                                                 (self.sendlen - self.sendidx))
                                    raise exceptions.TestFail("ThRecvCheck %s: "
                                                              "incorrect data" %
                                                              self.getName())
                    attempt = 10
                else:   # ! buf
                    # Broken socket
                    if attempt > 0:
                        attempt -= 1
                        if self.migrate_event is None:
                            self.exitevent.set()
                            raise exceptions.TestFail(
                                _err_msg_missing_migrate_ev)
                        logging.debug("ThRecvCheck %s: Broken pipe "
                                      ", reconnecting. ", self.getName())
                        self.reload_loss_idx()
                        # Wait until main thread sets the new self.port
                        while not (self.exitevent.isSet() or
                                   self.migrate_event.wait(1)):
                            pass
                        if self.exitevent.isSet():
                            break
                        logging.debug("ThRecvCheck %s: Broken pipe resumed, "
                                      "reconnecting...", self.getName())

                        self.port.sock = False
                        self.port.open()
        if self.sendidx >= 0:
            self.minsendidx = min(self.minsendidx, self.sendidx)
        if (self.sendlen - self.minsendidx):
            logging.error("ThRecvCheck %s: Data loss occurred during socket"
                          "reconnection. Maximal loss was %d per one "
                          "migration.", self.getName(),
                          (self.sendlen - self.minsendidx))
        logging.debug("ThRecvCheck %s: exit(%d)", self.getName(),
                      self.idx)
        self.ret_code = 0

    def run_debug(self):
        """
        viz run_normal.
        Additionally it stores last n verified characters and in
        case of failures it quickly receive enough data to verify failure or
        allowed loss and then analyze this data. It provides more info about
        the situation.
        Unlike normal run this one supports booth - loss and duplications.
        It's not friendly to data corruption.
        """
        logging.debug("ThRecvCheck %s: run", self.getName())
        attempt = 10
        max_loss = 0
        sum_loss = 0
        verif_buf = deque(maxlen=max(self.blocklen, self.sendlen))
        while not self.exitevent.isSet():
            ret = select.select([self.port.sock], [], [], 1.0)
            if ret[0] and (not self.exitevent.isSet()):
                buf = self.port.sock.recv(self.blocklen)
                if buf:
                    # Compare the received data with the control data
                    for idx_char in xrange(len(buf)):
                        _char = self.buff.popleft()
                        char = struct.pack('B', (bytearray(buf)[idx_char]))
                        if char == _char:
                            self.idx += 1
                            verif_buf.append(_char)
                        else:
                            # Detect the duplicated/lost characters.
                            logging.debug("ThRecvCheck %s: fail to receive "
                                          "%dth character.", self.getName(),
                                          self.idx)
                            buf = buf[idx_char:]
                            for i in xrange(100):
                                if len(self.buff) < self.sendidx:
                                    time.sleep(0.01)
                                else:
                                    break
                            sendidx = min(self.sendidx, len(self.buff))
                            if sendidx < self.sendidx:
                                logging.debug("ThRecvCheck %s: sendidx was "
                                              "lowered as there is not enough "
                                              "data after 1s. Using sendidx="
                                              "%s.", self.getName(), sendidx)
                            for _ in xrange(sendidx // self.blocklen):
                                if self.exitevent.isSet():
                                    break
                                buf += self.port.sock.recv(self.blocklen)
                            queue = _char
                            for _ in xrange(sendidx):
                                queue += self.buff[_]
                            offset_a = None
                            offset_b = None
                            for i in xrange(sendidx):
                                length = min(len(buf[i:]), len(queue))
                                if buf[i:] == queue[:length]:
                                    offset_a = i
                                    break
                            for i in xrange(sendidx):
                                length = min(len(queue[i:]), len(buf))
                                if queue[i:][:length] == buf[:length]:
                                    offset_b = i
                                    break

                            if (offset_b and offset_b < offset_a) or offset_a:
                                # Data duplication
                                self.sendidx -= offset_a
                                max_loss = max(max_loss, offset_a)
                                sum_loss += offset_a
                                logging.debug("ThRecvCheck %s: DUP %s (out of "
                                              "%s)", self.getName(), offset_a,
                                              sendidx)
                                buf = buf[offset_a + 1:]
                                for _ in xrange(len(buf)):
                                    self.buff.popleft()
                                verif_buf.extend(buf)
                                self.idx += len(buf)
                            elif offset_b:  # Data loss
                                max_loss = max(max_loss, offset_b)
                                sum_loss += offset_b
                                logging.debug("ThRecvCheck %s: LOST %s (out of"
                                              " %s)", self.getName(), offset_b,
                                              sendidx)
                                # Pop-out the lost characters from verif_queue
                                # (first one is already out)
                                self.sendidx -= offset_b
                                for i in xrange(offset_b - 1):
                                    self.buff.popleft()
                                for _ in xrange(len(buf)):
                                    self.buff.popleft()
                                self.idx += len(buf)
                                verif_buf.extend(buf)
                            else:   # Too big data loss or duplication
                                verif = ""
                                for _ in xrange(-min(sendidx, len(verif_buf)),
                                                0):
                                    verif += verif_buf[_]
                                logging.error("ThRecvCheck %s: mismatched data"
                                              ":\nverified: ..%s\nreceived:   "
                                              "%s\nsent:       %s",
                                              self.getName(), repr(verif),
                                              repr(buf), repr(queue))
                                raise exceptions.TestFail("Recv and sendqueue "
                                                          "don't match with any offset.")
                            # buf was changed, break from this loop
                            attempt = 10
                            break
                    attempt = 10
                else:   # ! buf
                    # Broken socket
                    if attempt > 0:
                        attempt -= 1
                        if self.migrate_event is None:
                            self.exitevent.set()
                            raise exceptions.TestFail("ThRecvCheck %s: Broken pipe."
                                                      " If this is expected behavior set migrate"
                                                      "_event to support reconnection." %
                                                      self.getName())
                        logging.debug("ThRecvCheck %s: Broken pipe "
                                      ", reconnecting. ", self.getName())
                        self.reload_loss_idx()
                        # Wait until main thread sets the new self.port
                        while not (self.exitevent.isSet() or
                                   self.migrate_event.wait(1)):
                            pass
                        if self.exitevent.isSet():
                            break
                        logging.debug("ThRecvCheck %s: Broken pipe resumed, "
                                      "reconnecting...", self.getName())

                        self.port.sock = False
                        self.port.open()
        if self.sendidx >= 0:
            self.minsendidx = min(self.minsendidx, self.sendidx)
        if (self.sendlen - self.minsendidx):
            logging.debug("ThRecvCheck %s: Data loss occurred during socket"
                          "reconnection. Maximal loss was %d per one "
                          "migration.", self.getName(),
                          (self.sendlen - self.minsendidx))
        if sum_loss > 0:
            logging.debug("ThRecvCheck %s: Data offset detected, cumulative "
                          "err: %d, max err: %d(%d)", self.getName(), sum_loss,
                          max_loss, float(max_loss) / self.blocklen)
        logging.debug("ThRecvCheck %s: exit(%d)", self.getName(),
                      self.idx)
        self.ret_code = 0



"""
Module simplifying manipulation of XML described at
http://libvirt.org/formatstorage.html#StorageVol
"""

from virttest.libvirt_xml import base, accessors
from virttest.libvirt_xml.xcepts import LibvirtXMLNotFoundError


class VolXMLBase(base.LibvirtXMLBase):

    """
    Accessor methods for VolXML class.

    Properties:
        name: string, operates on XML name tag
        key: string, operates on key tag
        capacity: integer, operates on capacity attribute of capacity tag
        allocation: integer, operates on allocation attribute of allocation
        format: string, operates on type attribute of format tag
        path: string, operates on path attribute of path tag
        owner, integer, operates on owner attribute of owner tag
        group, integer, operates on group attribute of group tag
        mode: string, operates on mode attribute of mode tag
        label: string, operates on label attribute of label tag
        compat: string, operates on compat attribute of label tag
        lazy_refcounts: bool, True/False
        encryption: VolXMLBase.Encryption instance.
        capacity_unit: string, operates on unit attribute of capacity tag
    """

    __slots__ = ('name', 'key', 'capacity', 'allocation', 'format', 'path',
                 'owner', 'group', 'mode', 'label', 'compat', 'lazy_refcounts',
                 'encryption', "capacity_unit")

    __uncompareable__ = base.LibvirtXMLBase.__uncompareable__

    __schema_name__ = "storagevol"

    def __init__(self, virsh_instance=base.virsh):
        accessors.XMLElementText('name', self, parent_xpath='/',
                                 tag_name='name')
        accessors.XMLElementText('key', self, parent_xpath='/',
                                 tag_name='key')
        accessors.XMLElementInt('capacity', self, parent_xpath='/',
                                tag_name='capacity')
        accessors.XMLElementInt('allocation', self, parent_xpath='/',
                                tag_name='allocation')
        accessors.XMLAttribute('format', self, parent_xpath='/target',
                               tag_name='format', attribute='type')
        accessors.XMLAttribute('capacity_unit', self, parent_xpath='/',
                               tag_name='capacity', attribute='unit')
        accessors.XMLElementNest('encryption', self, parent_xpath='/target',
                                 tag_name='encryption', subclass=self.Encryption,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementText('path', self, parent_xpath='/target',
                                 tag_name='path')
        accessors.XMLElementInt('owner', self,
                                parent_xpath='/target/permissions',
                                tag_name='owner')
        accessors.XMLElementInt('group', self,
                                parent_xpath='/target/permissions',
                                tag_name='group')
        accessors.XMLElementText('mode', self,
                                 parent_xpath='/target/permissions',
                                 tag_name='mode')
        accessors.XMLElementText('label', self,
                                 parent_xpath='/target/permissions',
                                 tag_name='label')
        accessors.XMLElementText('compat', self, parent_xpath='/target',
                                 tag_name='compat')
        accessors.XMLElementBool('lazy_refcounts', self,
                                 parent_xpath='/target/features',
                                 tag_name='lazy_refcounts')
        super(VolXMLBase, self).__init__(virsh_instance=virsh_instance)


class VolXML(VolXMLBase):

    """
    Manipulators of a Virtual Vol through it's XML definition.
    """

    __slots__ = []

    def __init__(self, vol_name='default', virsh_instance=base.virsh):
        """
        Initialize new instance with empty XML
        """
        super(VolXML, self).__init__(virsh_instance=virsh_instance)
        self.xml = u"<volume><name>%s</name></volume>" % vol_name

    def new_encryption(self, **dargs):
        """
        Return a new volume encryption instance and set properties from dargs
        """
        new_one = self.Encryption(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def create(self, pool_name, virsh_instance=base.virsh):
        """
        Create volume with virsh from this instance
        """
        result = virsh_instance.vol_create(pool_name, self.xml)
        if result.exit_status:
            return False
        return True

    @staticmethod
    def new_from_vol_dumpxml(vol_name, pool_name, virsh_instance=base.virsh):
        """
        Return new VolXML instance from virsh vol-dumpxml command

        :param vol_name: Name of vol to vol-dumpxml
        :param virsh_instance: virsh module or instance to use
        :return: New initialized VolXML instance
        """
        volxml = VolXML(virsh_instance=virsh_instance)
        result = virsh_instance.vol_dumpxml(vol_name, pool_name)
        volxml['xml'] = result.stdout_text.strip()
        return volxml

    @staticmethod
    def get_vol_details_by_name(vol_name, pool_name, virsh_instance=base.virsh):
        """
        Return volume xml dictionary by Vol's uuid or name.

        :param vol_name: Vol's name
        :return: volume xml dictionary
        """
        volume_xml = {}
        vol_xml = VolXML.new_from_vol_dumpxml(vol_name, pool_name,
                                              virsh_instance)
        volume_xml['key'] = vol_xml.key
        volume_xml['path'] = vol_xml.path
        volume_xml['capacity'] = vol_xml.capacity
        volume_xml['allocation'] = vol_xml.allocation
        try:
            volume_xml['format'] = vol_xml.format
        except LibvirtXMLNotFoundError:
            volume_xml['format'] = None
        return volume_xml

    @staticmethod
    def new_vol(**dargs):
        """
        Return a new VolXML instance and set properties from dargs

        :param dargs: param dictionary
        :return: new VolXML instance
        """
        new_one = VolXML(virsh_instance=base.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    class Encryption(base.LibvirtXMLBase):

        """
        Encryption volume XML class

        Properties:

        format:
            string.
        secret:
            dict, keys: type, uuid
        """

        __slots__ = ('format', 'secret')

        def __init__(self, virsh_instance=base.virsh):
            accessors.XMLAttribute('format', self, parent_xpath='/',
                                   tag_name='encryption', attribute='format')
            accessors.XMLElementDict('secret', self, parent_xpath='/',
                                     tag_name='secret')
            super(VolXML.Encryption, self).__init__(
                virsh_instance=virsh_instance)
            self.xml = '<encryption/>'



#!/usr/bin/python
"""
Based on work from Douglas Creager <dcreager@dcreager.net>

Gets the current version number.  If possible, this is the
output of "git describe", modified to conform to the versioning
scheme that setuptools uses.  If "git describe" returns an error
(most likely because we're in an unpacked copy of a release tarball,
rather than in a git working copy), then we fall back on reading the
contents of the RELEASE-VERSION file.
"""
__all__ = ("get_git_version", "get_version", "get_top_commit",
           "get_current_branch", "get_pretty_version_info")

import os

from avocado.utils import process

from virttest import data_dir

_ROOT_PATH = data_dir.get_root_dir()
RELEASE_VERSION_PATH = os.path.join(_ROOT_PATH, 'RELEASE-VERSION')

global _GIT_VERSION_CACHE, _VERSION_CACHE, _TOP_COMMIT_CACHE
global _CURRENT_BRANCH_CACHE, _PRETTY_VERSION_CACHE

_GIT_VERSION_CACHE = None
_VERSION_CACHE = None
_TOP_COMMIT_CACHE = None
_CURRENT_BRANCH_CACHE = None
_PRETTY_VERSION_CACHE = None


def _execute_git_command(command):
    """
    As git is sensitive to the $CWD, change to the top dir to execute git cmds.

    :param: command - Git command to be executed.
    """
    cwd = os.getcwd()
    os.chdir(_ROOT_PATH)
    try:
        try:
            return process.run(command,
                               shell=True, verbose=False).stdout_text.strip()
        finally:
            os.chdir(cwd)
    except process.CmdError:
        return None


def get_git_version(abbrev=4):
    global _GIT_VERSION_CACHE
    if _GIT_VERSION_CACHE is not None:
        return _GIT_VERSION_CACHE

    _GIT_VERSION_CACHE = _execute_git_command('git describe --abbrev=%d' %
                                              abbrev)

    return _GIT_VERSION_CACHE


def get_top_commit():
    global _TOP_COMMIT_CACHE
    if _TOP_COMMIT_CACHE is not None:
        return _TOP_COMMIT_CACHE

    _TOP_COMMIT_CACHE = _execute_git_command(
        "git show --summary --pretty='%H' | head -1")

    return _TOP_COMMIT_CACHE


def get_current_branch():
    global _CURRENT_BRANCH_CACHE
    if _CURRENT_BRANCH_CACHE is not None:
        return _CURRENT_BRANCH_CACHE
    _CURRENT_BRANCH_CACHE = _execute_git_command('git rev-parse '
                                                 '--abbrev-ref HEAD')
    return _CURRENT_BRANCH_CACHE


def _read_release_version():
    try:
        f = open(RELEASE_VERSION_PATH, "r")
        try:
            version = f.readlines()[0]
            return version.strip()
        finally:
            f.close()
    except Exception:
        return None


def _write_release_version(version):
    try:
        f = open(RELEASE_VERSION_PATH, "w")
        f.write("%s\n" % version)
        f.close()
    except Exception:
        pass


def get_version(abbrev=4):
    global _GIT_VERSION_CACHE
    release_version = _read_release_version()
    if _GIT_VERSION_CACHE is not None:
        version = _GIT_VERSION_CACHE
    else:
        _GIT_VERSION_CACHE = get_git_version(abbrev)
        version = _GIT_VERSION_CACHE

    if version is None:
        version = release_version

    if version is None:
        try:
            cmd_result = process.run("rpm -q avocado-plugins-vt "
                                     "--queryformat '%{VERSION}'",
                                     shell=True, verbose=False)
            return '%s (RPM install)' % cmd_result.stdout_text
        except process.CmdError:
            return 'unknown'

    if version != release_version:
        _write_release_version(version)

    return version


def get_pretty_version_info():
    version_str = "Avocado-VT '%s'" % get_version()
    current_branch = get_current_branch()
    if current_branch:
        version_str += ", Branch '%s'" % current_branch
    top_commit = get_top_commit()
    if top_commit:
        version_str += ", SHA1 '%s'" % top_commit
    return version_str


if __name__ == "__main__":
    print(get_pretty_version_info())



import os
import logging
import threading
import functools
try:
    from collections import UserDict as IterableUserDict
except ImportError:
    from UserDict import IterableUserDict
try:
    import pickle as cPickle
except ImportError:
    import cPickle

from aexpect import remote

from avocado.core import exceptions

from virttest import virt_vm
from virttest import ip_sniffing

ENV_VERSION = 1


def get_env_version():
    return ENV_VERSION


class EnvSaveError(Exception):
    pass


def lock_safe(function):
    """
    Get the environment safe lock, run the function, then release the lock.

    Unfortunately, it only works if the 1st argument of the function is an
    Env instance. This is mostly to save up code.

    :param function: Function to wrap.
    """
    @functools.wraps(function)
    def wrapper(env, *args, **kwargs):
        with env.save_lock:
            return function(env, *args, **kwargs)
    return wrapper


class Env(IterableUserDict):

    """
    A dict-like object containing global objects used by tests.
    """

    def __init__(self, filename=None, version=0):
        """
        Create an empty Env object or load an existing one from a file.

        If the version recorded in the file is lower than version, or if some
        error occurs during unpickling, or if filename is not supplied,
        create an empty Env object.

        :param filename: Path to an env file.
        :param version: Required env version (int).
        """
        IterableUserDict.__init__(self)
        empty = {"version": version}
        self._filename = filename
        self._sniffer = None
        self.save_lock = threading.RLock()
        if filename:
            try:
                if os.path.isfile(filename):
                    with open(filename, "rb") as f:
                        env = cPickle.load(f)
                    if env.get("version", 0) >= version:
                        self.data = env
                    else:
                        logging.warn(
                            "Incompatible env file found. Not using it.")
                        self.data = empty
                else:
                    # No previous env file found, proceed...
                    logging.warn("Creating new, empty env file")
                    self.data = empty
            # Almost any exception can be raised during unpickling, so let's
            # catch them all
            except Exception as e:
                logging.warn("Exception thrown while loading env")
                logging.warn(e)
                logging.warn("Creating new, empty env file")
                self.data = empty
        else:
            logging.warn("Creating new, empty env file")
            self.data = empty

    def save(self, filename=None):
        """
        Pickle the contents of the Env object into a file.

        :param filename: Filename to pickle the dict into.  If not supplied,
                use the filename from which the dict was loaded.
        """
        filename = filename or self._filename
        if filename is None:
            raise EnvSaveError("No filename specified for this env file")
        with self.save_lock, open(filename, "wb") as f:
            cPickle.dump(self.data, f, protocol=0)

    def get_all_vms(self):
        """
        Return a list of all VM objects in this Env object.
        """
        return [v for k, v in self.data.items() if k and k.startswith("vm__")]

    def clean_objects(self):
        """
        Destroy all objects registered in this Env object.
        """
        self.stop_ip_sniffing()
        for key in self.data:
            try:
                if key.startswith("vm__"):
                    self.data[key].destroy(gracefully=False)
            except Exception:
                pass
        self.data = {}

    def destroy(self):
        """
        Destroy all objects stored in Env and remove the backing file.
        """
        self.clean_objects()
        if self._filename is not None:
            if os.path.isfile(self._filename):
                os.unlink(self._filename)

    def get_vm(self, name):
        """
        Return a VM object by its name.

        :param name: VM name.
        """
        return self.data.get("vm__%s" % name)

    def create_vm(self, vm_type, target, name, params, bindir):
        """
        Create and register a VM in this Env object
        """
        vm_class = virt_vm.BaseVM.lookup_vm_class(vm_type, target)
        if vm_class is not None:
            vm = vm_class(name, params, bindir, self.get("address_cache"))
            self.register_vm(name, vm)
            return vm

    @lock_safe
    def register_vm(self, name, vm):
        """
        Register a VM in this Env object.

        :param name: VM name.
        :param vm: VM object.
        """
        self.data["vm__%s" % name] = vm

    @lock_safe
    def unregister_vm(self, name):
        """
        Remove a given VM.

        :param name: VM name.
        """
        del self.data["vm__%s" % name]

    @lock_safe
    def register_syncserver(self, port, server):
        """
        Register a Sync Server in this Env object.

        :param port: Sync Server port.
        :param server: Sync Server object.
        """
        self.data["sync__%s" % port] = server

    @lock_safe
    def unregister_syncserver(self, port):
        """
        Remove a given Sync Server.

        :param port: Sync Server port.
        """
        del self.data["sync__%s" % port]

    def get_syncserver(self, port):
        """
        Return a Sync Server object by its port.

        :param port: Sync Server port.
        """
        return self.data.get("sync__%s" % port)

    @lock_safe
    def register_lvmdev(self, name, lvmdev):
        """
        Register lvm device object into env;

        :param name: name of register lvmdev object
        :param lvmdev: lvmdev object;
        """
        self.data["lvmdev__%s" % name] = lvmdev

    @lock_safe
    def unregister_lvmdev(self, name):
        """
        Remove lvm device object from env;

        :param name: name of lvm device object;
        """
        del self.data["lvmdev__%s" % name]

    def get_lvmdev(self, name):
        """
        Get lvm device object by name from env;

        :param name: lvm device object name;
        :return: lvmdev object
        """
        return self.data.get("lvmdev__%s" % name)

    def start_ip_sniffing(self, params):
        """
        Start ip sniffing.

        :param params: Params object.
        """
        self.data.setdefault("address_cache", ip_sniffing.AddrCache())
        sniffers = ip_sniffing.Sniffers

        if not self._sniffer:
            remote_pp = params.get("remote_preprocess") == "yes"
            remote_opts = None
            session = None
            if remote_pp:
                client = params.get('remote_shell_client', 'ssh')
                remote_opts = (params['remote_node_address'],
                               params.get('remote_shell_port', '22'),
                               params['remote_node_user'],
                               params['remote_node_password'],
                               params.get('remote_shell_prompt', '#'))
                session = remote.remote_login(client, *remote_opts)
            for s_cls in sniffers:
                if s_cls.is_supported(session):
                    self._sniffer = s_cls(self.data["address_cache"],
                                          "ip-sniffer.log",
                                          remote_opts)
                    break
            if session:
                session.close()

        if not self._sniffer:
            raise exceptions.TestError("Can't find any supported ip sniffer! "
                                       "%s" % [s.command for s in sniffers])

        self._sniffer.start()

    def stop_ip_sniffing(self):
        """Stop ip sniffing."""
        if self._sniffer:
            self._sniffer.stop()



import operator
import re
from distutils.version import LooseVersion  # pylint: disable=no-name-in-module,import-error


class VersionInterval(object):
    """
    A class for a Version Interval object.

    An interval is a string representation of a mathmetical like interval.
    e.g: "(3,4]", "[3.10.0,)"
    A verison is a version string.

    Examples::
        >>> verison_interval = VersionInterval("[2.10.1, 2.11.0)")
        >>> verison = "2.10.16"
        >>> verison in verison_interval
        True
        >>> verison = "2.13.0"
        >>> verison in verison_interval
        False

    """

    def __init__(self, interval):
        interval_rex = r"^(\[|\()(.*?)\s*,\s*(.*?)(\]|\))$"
        match = re.search(interval_rex, interval)
        if match is None:
            raise ValueError("Invaild string representation of an interval")
        self.opening, lower, upper, self.closing = match.groups()

        self.lower_bound = LooseVersion(lower) if lower else None
        self.upper_bound = LooseVersion(upper) if upper else None
        self._check_interval()

    def _check_interval(self):
        if not (self.upper_bound and self.lower_bound):
            return
        if self.lower_bound < self.upper_bound:
            return
        if (self.lower_bound == self.upper_bound and self.opening == '[' and
                self.closing == ']'):
            return
        raise ValueError("Invaild interval")

    def __repr__(self):
        return '<version interval %s%s, %s%s>' % (self.opening,
                                                  self.lower_bound,
                                                  self.upper_bound,
                                                  self.closing)

    def __contains__(self, version):
        op_mapping = {"(": operator.lt, "[": operator.le,
                      ")": operator.gt, "]": operator.ge}
        in_interval = True
        version = LooseVersion(version)
        if self.lower_bound:
            opt = op_mapping.get(self.opening)
            in_interval = opt(self.lower_bound, version)
        if in_interval and self.upper_bound:
            opt = op_mapping.get(self.closing)
            in_interval = opt(self.upper_bound, version)
        return in_interval
